/****************************
  The MIT License (MIT)

  Copyright (c) 2015 by bbx10node@gmail.com

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
******************************/

#include <ESP8266WiFi.h>

#include <ArduinoJson.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>

#include <ArduinoJson.h>

#include <Adafruit_NeoPixel.h>

MDNSResponder mdns;

const char* ssid = "FAB-WIFI";
const char* password = "barcelona2007";
static char respBuf[4096];

long previousMillis = 0;
long interval = 10000;           // interval at which to blink (milliseconds)

boolean connectWeather = false;
int count = 0;
float temp_c = 0.0;
char icon[100] ;
float humi = 0.0;
char weather[100] ;

#define WU_API_KEY "f0b476296999184b"
#define WU_LOCATION "SPAIN/Barcelona"

#define LEDPIN 2 // Data pin for LED

#define NUM_LEDS 16 //Number of LEDs
int red = 100;
int green = 0;
int blue = 0;
Adafruit_NeoPixel pixels = Adafruit_NeoPixel (NUM_LEDS, LEDPIN, NEO_GRB + NEO_KHZ800);

// 5 minutes between update checks. The free developer account has a limit
// on the  number of calls so don't go wild.
#define DELAY_NORMAL    (5*60*1000)
// 20 minute delay between updates after an error
#define DELAY_ERROR     (20*60*1000)

#define WUNDERGROUND "api.wunderground.com"

// HTTP request
const char WUNDERGROUND_REQ[] =
  "GET /api/" WU_API_KEY "/conditions/q/" WU_LOCATION ".json HTTP/1.1\r\n"
  "User-Agent: ESP8266/0.1\r\n"
  "Accept: */*\r\n"
  "Host: " WUNDERGROUND "\r\n"
  "Connection: close\r\n"
  "\r\n";

ESP8266WebServer server(80);

String webPage = "";

void setup(void) {

  count = 0;


  delay(1000);
  Serial.begin(115200);
  WiFi.begin(ssid, password);
  Serial.println("");

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(F("."));
  }

  Serial.println();
  Serial.println(F("Connected to"));
  Serial.println(F("ssid"));
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  if (mdns.begin("esp8266", WiFi.localIP())) {
    Serial.println("MDNS responder started");
  }

  server.on("/", []() {
    Serial.print("CONNECTED TO CLOUDIO");
    server.send(200, "text/html", webPage);
  });

  server.begin();
  Serial.println("HTTP server started");
      
      delay(2000);
pixels.begin();
      delay(2000);

}

void loop(void)
{
  server.handleClient();

  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis > interval) {
    // save the last time you blinked the LED
    previousMillis = currentMillis;

    Serial.print("Connecting to ");
    Serial.println(WUNDERGROUND);

    // Use WiFiClient class to create TCP connections
    WiFiClient httpclient;
    const int httpPort = 80;
    if (!httpclient.connect(WUNDERGROUND, httpPort)) {
      Serial.println(F("connection failed"));
      delay(DELAY_ERROR);
      return;
    }

    // This will send the http request to the server
    Serial.print(WUNDERGROUND_REQ);
    httpclient.print(WUNDERGROUND_REQ);
    httpclient.flush();

    // Collect http response headers and content from Weather Underground
    // HTTP headers are discarded.
    // The content is formatted in JSON and is left in respBuf.
    int respLen = 0;
    bool skip_headers = true;
    while (httpclient.connected() || httpclient.available()) {
      if (skip_headers) {
        String aLine = httpclient.readStringUntil('\n');
        //Serial.println(aLine);
        // Blank line denotes end of headers
        if (aLine.length() <= 1) {
          skip_headers = false;
        }
      }
      else {
        int bytesIn;
        bytesIn = httpclient.read((uint8_t *)&respBuf[respLen], sizeof(respBuf) - respLen);
        Serial.print(F("bytesIn ")); Serial.println(bytesIn);
        if (bytesIn > 0) {
          respLen += bytesIn;
          if (respLen > sizeof(respBuf)) respLen = sizeof(respBuf);
        }
        else if (bytesIn < 0) {
          Serial.print(F("read error "));
          Serial.println(bytesIn);
        }
      }
      delay(1);
    }
    httpclient.stop();

    if (respLen >= sizeof(respBuf)) {
      Serial.print(F("respBuf overflow "));
      Serial.println(respLen);
      delay(DELAY_ERROR);
      return;
    }
    // Terminate the C string
    respBuf[respLen++] = '\0';
    Serial.print(F("respLen "));
    Serial.println(respLen);
    //Serial.println(respBuf);
    webPage = "<html> <head><link href='https://fonts.googleapis.com/css?family=Roboto:100,300,400,500,700' rel='stylesheet'> <style>body { background: #9aebff; font-family: 'Roboto', sans-serif; font-size: 14px; line-height: 1.5; margin: 0 auto; max-width: 800px; padding: 2em 2em 4em; text-align: center; content: center; align-items: center; }h1, h2, h3, h4, h5, h6 { line-height: 1.3; margin: auto; text-align: center; text-decoration-line: none; } h1 { font-weight: 100; color: #ffffff; font-family: 'Abel', sans-serif; margin-top: 1.3em; font-size: 60px;} h2 { color: #ffffff; font-weight: 300; margin-top: 1.3em; font-size: 20px; } h3 { margin-top: 1.3em; font-size: 20px;} </style> </head> <body> <h1>cloudio</h1> <img src='" + String(icon) + "'> <br><br>Temperature Outdoor:&nbsp;" + String(temp_c) + "<br>Humidity Outdoor:&nbsp;" + String(humi) + "<br>" + String(weather) + "<br></h2> </body></html>";


    showWeather(respBuf);

    connectWeather = true;

    for (int i = 0; i < NUM_LEDS; i++) {
      pixels.setPixelColor(i, pixels.Color(red, green, blue)); // Moderately bright green color.
    } pixels.show();

        if (temp_c > 23) {
      setForecast(161, 218, 8);

}
if (temp_c > 24) {
      setForecast(164, 255, 194);

}
    if (temp_c > 25) {
      setForecast(189, 239, 255);

}
      if (temp_c > 26) {
        setForecast(94, 244, 225);
      }
      if (temp_c > 27) {
        setForecast(126, 244, 94);
      }

      if (temp_c > 28) {
        setForecast(244, 221, 94);
      }

      if (temp_c > 29) {
        setForecast(6, 204, 160);
      }
      if (temp_c > 30) {
        setForecast(6, 178, 204);
      }
      if (temp_c < 30) {
        setForecast(177,124, 255);
      }
      if (temp_c < 31) {
        setForecast(188, 124, 0);
      }
      if (temp_c < 32) {
        setForecast(188, 204, 6);
      }
      if (temp_c < 33) {
        setForecast(251, 116, 17);
      }
      if (temp_c < 34) {
        setForecast(24, 204, 6);
      }
       if (temp_c < 35) {
        setForecast(131, 12, 3);
      }
        if (temp_c < 36) {
        setForecast(106, 31, 1);
      }

    }
  }


  void showWeather(char *json)
  {
    StaticJsonBuffer<3 * 1024> jsonBuffer;

    // Skip characters until first '{' found
    // Ignore chunked length, if present
    char *jsonstart = strchr(json, '{');
    //Serial.print(F("jsonstart ")); Serial.println(jsonstart);
    if (jsonstart == NULL) {
      Serial.println(F("JSON data missing"));
      //return false;
    }
    json = jsonstart;

    // Parse JSON
    JsonObject& root = jsonBuffer.parseObject(json);
    if (!root.success()) {
      Serial.println(F("jsonBuffer.parseObject() failed"));
      //return false;
    }

    // Extract weather info from parsed JSON
    JsonObject& current = root["current_observation"];

    temp_c = current["temp_c"];
    Serial.print(temp_c, 1); Serial.print(F(" C, "));
    humi = current[F("relative_humidity")];
    Serial.print(humi);   Serial.println(F(" RH"));

    strcpy(weather, current["weather"]);
    Serial.println(weather);

    strcpy(icon, current["icon_url"]);
    //icon = (current["icon_url"]);
    Serial.println(icon);

    const char *observation_time = current["observation_time_rfc822"];
    Serial.println(observation_time);

    // Extract local timezone fields
    const char *local_tz_short = current["local_tz_short"];
    Serial.println(local_tz_short);

    const char *local_tz_offset = current["local_tz_offset"];
    Serial.println(local_tz_offset);
  }

  void setForecast(int r, int g, int b) {
    red = r;
    green = g;
    blue = b;
  }
